class Version:
    def __init__(self, version=None):
        if version is None:
            self.patch = 0
            self.minor = 0
            self.major = 0

        elif isinstance(version, str):
            if "v" in version:
                version = version.replace("v", "")
            levels = version.split(".")
            self.major = int(levels[0])
            self.minor = int(levels[1])
            self.patch = int(levels[2])

    def to_str(self):
        version_str = (
            "v" + str(self.major) + "." + str(self.minor) + "." + str(self.patch)
        )
        return version_str

    def to_num():
        version_num = str(self.major) + "." + str(self.minor) + "." + str(self.patch)
        return version_num

    def bump(self, level="patch"):
        if level == "patch":
            self.patch += 1

        if level == "minor":
            self.minor += 1
            self.patch = 0

        if level == "major":
            self.major += 1
            self.minor = 0
            self.patch = 0



import json

#TODO: Eventually break out associated info (URLS) into a new class

class RepositoryMetadata(object):
    def __init__(self, urls, permissions) -> None:
        self.repositoryURL = urls[0]
        self.homepageURL = urls[1]
        self.downloadURL = urls[2]
        self.disclaimerURL = urls[3]
        self.licenseURL = self.permissions["licenses"]["URL"]


import sys
import json
import copy
from pathlib import Path
import typer

# Local imports
from esi_releases.format import release_metadata
from esi_releases.utils.metadata_utils import load_code_json
from esi_releases.utils.metadata_utils import BASE_DIR, DATA_DIR, TEST_DATA_DIR

project_code_json = BASE_DIR / "code.json"
example_code_json = DATA_DIR / "example_code.json"
default_code_json = DATA_DIR / "default_code.json"

app = typer.Typer()


@app.command()
def bump_version(
    release_level: str = typer.Argument(
        default="patch", help="Version level to bump at i.e. 'major'.'minor'.'patch'"
    )
):
    # major: str = (typer.Option(None, help="Bump version at the major level aka N.0.0"),)
    # minor: str = (typer.Option(None, help="Bump version at the minor level aka 0.N.0"),)
    # patch: str = (typer.Option(None, help="Bump version at the patch level aka 0.0.N"),)

    try:
        # Load project code.json file
        code_json = load_code_json(project_code_json)

        # Instantiate new release_metadata obj
        current_release = release_metadata.ReleaseMetadata()
        current_release.fill_from_json(project_code_json)

        # Can improve if using above list
        new_release = copy.deepcopy(current_release)
        new_release.bump_version(release_level=release_level)

        # Update the "metadataLastUpdated" field to current time
        new_release.sync_time()

        # Prepend new release metadata and convert to JSON str
        bumped_release = [new_release.__dict__] + code_json
        bumped_release_json = json.dumps(bumped_release, indent=4)

        # Write to the code.json file
        with open(project_code_json, mode="w") as outfile:
            outfile.write(bumped_release_json)
    except Exception as e:
        return e


# def main():
#     new_release = release_metadata.ReleaseMetadata()


if __name__ == "__main__":
    # app()
    typer.run(bump_version)



import sys
import json
from pathlib import Path

import pytest

# from esi_releases.format import release_metadata

BASE_DIR = (Path(".").parent).resolve()
DATA_DIR = (Path(".").parent / "src" / "esi_releases" / "data").resolve()
TEST_DATA_DIR = (Path(".").parent / "tests" / "data").resolve()

JSON_FIELDS = [
    "name",
    "description",
    "version",
    "status",
    "organization",
    "vcs",
    "repositoryURL",
    "homepageURL",
    "downloadURL",
    "disclaimerURL",
    "permissions",
    "laborHours",
    "languages",
    "tags",
    "contact",
    "date",
]

JSON_URL_FIELDS = [
    "repositoryURL",
    "homepageURL",
    "downloadURL",
    "disclaimerURL",
    "permissions",
]

JSON_PKG_VERSION_FIELDS = [
    "downloadURL",
    "disclaimerURL",
    "permissions",
]


def load_code_json(filename):
    if not isinstance(filename, str):
        filename = str(filename)

    with open(filename, "r") as f:
        try:
            print("Loading json file: " + filename)
            code_json = json.load(f)

            return code_json

        except Exception as e:
            raise e


# def check_version_format():
#     # Check if version format includes a prepended 'v' or not
#     return


# def check_release_history():
#     # Print out all previous versions from code.json
#     return


# def validate_release_history():
#     # Check that release history entries are increasing in version #
#     return



import re
import json
from pathlib import Path
from datetime import datetime

from esi_releases.utils.metadata_utils import (
    JSON_FIELDS,
    JSON_PKG_VERSION_FIELDS,
    DATA_DIR,
    TEST_DATA_DIR,
)
from esi_releases.format.version import Version

example_code_json = DATA_DIR / "example_code.json"


class ReleaseMetadata:
    def __init__(self, version="v0.0.0") -> None:
        self.name = "usgs-package"
        self.description = "Description of functionality"
        self.version = str(version)
        self.status = "Development"
        self.organization = "U.S. Geological Survey"
        self.vcs = "git"
        self.repositoryURL = "https://code.usgs.gov/ghsc/esi/my-cool-package/.git"
        self.homepageURL = "https://code.usgs.gov/ghsc/esi/my-cool-packages/"
        self.downloadURL = ""
        self.disclaimerURL = {
            "https://code.usgs.gov/ghsc/esi/my-cool-package/-/tree/main/DISCLAIMER.md"
        }
        self.permissions = (
            {
                "licenses": {
                    "name": "Public Domain, CC0-1.0",
                    "URL": "https://code.usgs.gov/ghsc/esi/my-cool-package/-/tree/main/LICENSE.md",
                },
                "usageType": "openSource",
                "exemptionText": "null",
            },
        )
        self.laborHours = 1
        self.languages = ["python", "bash"]
        self.tags = [
            "releases",
            "utilities",
        ]
        self.contact = {"name": " Name Name", "email": "name@usgs.gov"}
        self.date = {"metadataLastUpdated": str(datetime.now()).split(" ", 1)[0]}

    def get_name(self):
        return self.name

    def set_name(self, name):
        self.name = name

    def get_description(self):
        return self.name

    def set_description(self, description):
        self.description = description

    def get_version(
        self,
    ):
        return self.version

    def set_version(self, version):
        if "v" not in version:
            version = "v" + str(version)
        self.version = version

    def get_URLs(self):
        url_dict = {
            "repositoryURL": self.repositoryURL,
            "homepageURL": self.homepageURL,
            "downloadURL": self.downloadURL,
            "disclaimerURL": self.disclaimerURL,
            "licenseURL": self.permissions["licenses"][0]["URL"],
        }

        return url_dict

    def set_URLs(self, urls):
        self.repositoryURL = urls[0]
        self.homepageURL = urls[1]
        self.downloadURL = urls[2]
        self.disclaimerURL = urls[3]
        self.licenseURL = self.permissions["licenses"]["URL"]

    def update_URLs(self):
        # Update the version tags in relevant URLs
        version = self.get_version()

        old_URLs = self.get_URLs()

        REGEX_V = r"v[0-9]+\.[0-9]+\.[0-9]+"
        # REGEX_V = r"[0-9]+\.[0-9]+\.[0-9]+"

        new_urls = old_URLs.copy()
        for url in old_URLs:
            new_urls[url] = re.sub(REGEX_V, version, old_URLs[url], re.IGNORECASE)
            # print(new_urls)

        self.downloadURL = new_urls["downloadURL"]
        self.disclaimerURL = new_urls["disclaimerURL"]
        self.licenseURL = new_urls["licenseURL"]
        self.permissions["licenses"][0]["URL"] = new_urls["licenseURL"]

    def get_metadata_fields(self):
        return JSON_FIELDS

    def fill_from_json(self, filename):
        filename = str(filename)

        with open(filename, "r") as f:
            try:
                print("json filename: " + filename)
                code_json = json.load(f)

                # Get most recent release entry in code.json file
                self.__dict__.update(code_json[0])

            except Exception as e:
                raise e

    def bump_version(self, release_level="patch"):
        old_version = self.get_version()

        new_version = Version(version=old_version)
        new_version.bump(level=release_level)

        self.set_version(new_version.to_str())
        self.update_URLs()

    # def bump_URLs(self):

    def sync_time(self):
        self.date = {"metadataLastUpdated": str(datetime.now()).split(" ", 1)[0]}
